@@ -51,6 +51,7 @@
 static void CC_lookup_pg_version(ConnectionClass *self);
 static void CC_lookup_lo(ConnectionClass *self);
 static char *CC_create_errormsg(ConnectionClass *self);
+static int  CC_close_eof_cursors(ConnectionClass *self);
 
 extern GLOBAL_VALUES globals;
 
@@ -148,12 +149,14 @@ PGAPI_Connect(
 
 	qlog("conn = %p, %s(DSN='%s', UID='%s', PWD='%s')\n", conn, func, ci->dsn, ci->username, ci->password ? "xxxxx" : "");
 
-	if (CC_connect(conn, AUTH_REQ_OK, NULL) <= 0)
+	if ((fchar = CC_connect(conn, AUTH_REQ_OK, NULL)) <= 0)
 	{
 		/* Error messages are filled in */
 		CC_log_error(func, "Error on CC_connect", conn);
 		ret = SQL_ERROR;
 	}
+	if (SQL_SUCCESS == ret && 2 == fchar)
+		ret = SQL_SUCCESS_WITH_INFO;
 
 	mylog("%s: returning..%d.\n", func, ret);
 
@@ -444,6 +447,7 @@ CC_clear_error(ConnectionClass *self)
 
 
 CSTR	bgncmd = "BEGIN";
+CSTR	cmtcmd = "COMMIT";
 CSTR	rbkcmd = "ROLLBACK";
 CSTR	semi_colon = ";";
 /*
@@ -475,10 +479,15 @@ CC_commit(ConnectionClass *self)
 	char	ret = TRUE;
 	if (CC_is_in_trans(self))
 	{
-		QResultClass *res = CC_send_query(self, "COMMIT", NULL, 0, NULL);
-		mylog("CC_commit:  sending COMMIT!\n");
-		ret = QR_command_maybe_successful(res);
-		QR_Destructor(res);
+		if (!CC_is_in_error_trans(self))
+			CC_close_eof_cursors(self);
+		if (CC_is_in_trans(self))
+		{
+			QResultClass *res = CC_send_query(self, cmtcmd, NULL, 0, NULL);
+			mylog("CC_commit:  sending COMMIT!\n");
+			ret = QR_command_maybe_successful(res);
+			QR_Destructor(res);
+		}
 	}
 
 	return ret;
@@ -780,7 +789,7 @@ handle_error_message(ConnectionClass *self, char *msgbuf, size_t buflen, char *s
 
 			mylog("peek the next byte = \\0\n");
 			new_format = TRUE;
-			strncpy(ci->protocol, PG74, sizeof(ci->protocol));
+			strncpy_null(ci->protocol, PG74, sizeof(ci->protocol));
 			leng = SOCK_get_response_length(sock);
 			inolog("get the response length=%d\n", leng);
 		}
@@ -833,7 +842,7 @@ inolog("new_format=%d\n", new_format);
 					break;
 				case 'C':
 					if (sqlstate)
-						strncpy(sqlstate, msgbuffer + 1, 8);
+						strncpy_null(sqlstate, msgbuffer + 1, 8);
 					break;
 			}
 			if (buflen < 0)
@@ -944,7 +953,7 @@ handle_notice_message(ConnectionClass *self, char *msgbuf, size_t buflen, char *
 					break;
 				case 'C':
 					if (sqlstate && !sqlstate[0] && strcmp(msgbuffer + 1, "00000"))
-						strncpy(sqlstate, msgbuffer + 1, 8);
+						strncpy_null(sqlstate, msgbuffer + 1, 8);
 					break;
 			}
 			if (buflen < 0)
@@ -1015,7 +1024,7 @@ inolog("parameter name=%s\n", msgbuffer);
 		int	major, minor;
 
 		SOCK_get_string(sock, msgbuffer, sizeof(msgbuffer));
-		strncpy(conn->pg_version, msgbuffer, sizeof(conn->pg_version));
+		strncpy_null(conn->pg_version, msgbuffer, sizeof(conn->pg_version));
 		strcpy(szVersion, "0.0");
 		if (sscanf(conn->pg_version, "%d.%d", &major, &minor) >= 2)
 		{
@@ -1070,9 +1079,38 @@ static int	protocol3_opts_array(ConnectionClass *self, const char *opts[][2], BO
 	}
 	if (libpqopt)
 	{
-		if (ci->sslmode[0])
+		switch (ci->sslmode[0])
 		{
-			opts[cnt][0] = "sslmode";	opts[cnt++][1] = ci->sslmode;
+			case '\0':
+				break;
+			case SSLLBYTE_VERIFY:
+				opts[cnt][0] = "sslmode";
+				if (ssl_verify_available())
+				{
+					switch (ci->sslmode[1])
+					{
+						case 'f':
+							opts[cnt++][1] = SSLMODE_VERIFY_FULL;
+								break;
+						case 'c':
+							opts[cnt++][1] = SSLMODE_VERIFY_CA;
+								break;
+						default:
+							opts[cnt++][1] = ci->sslmode;
+					}
+				}
+				else
+				{
+					char msg[128];
+
+					snprintf(msg, sizeof(msg), "sslmode=%s can't be specified for 8.3 or older version of libpq", ci->sslmode);
+					CC_set_error(self, CONN_NOT_IMPLEMENTED_ERROR, msg, __FUNCTION__); 
+					return -1;
+				}
+				break;
+			default:
+				opts[cnt][0] = "sslmode";
+				opts[cnt++][1] = ci->sslmode;
 		}
 		if (ci->password[0])
 		{
@@ -1114,6 +1152,8 @@ static int	protocol3_packet_build(ConnectionClass *self)
 	int	cnt, i;
 
 	cnt = protocol3_opts_array(self, opts, FALSE, sizeof(opts) / sizeof(opts[0]));
+	if (cnt < 0)
+		return 0;
 
 	slen =  sizeof(ProtocolVersion);
 	for (i = 0; i < cnt; i++)
@@ -1166,6 +1206,8 @@ static char	*protocol3_opts_build(ConnectionClass *self)
 	BOOL	blankExist;
 
 	cnt = protocol3_opts_array(self, opts, TRUE, sizeof(opts) / sizeof(opts[0]));
+	if (cnt < 0)
+		return NULL;
 
 	slen =  sizeof(ProtocolVersion);
 	for (i = 0, slen = 0; i < cnt; i++)
@@ -1366,14 +1408,15 @@ original_CC_connect(ConnectionClass *self, char password_req, char *salt_para)
 		ssl_try_count = 0;
 		switch (self->connInfo.sslmode[0])
 		{
-			case 'r':
+			case SSLLBYTE_REQUIRE:
+			case SSLLBYTE_VERIFY:
 				ssl_call[ssl_try_count++] = 'y';
 				break;
-			case 'p':
+			case SSLLBYTE_PREFER:
 				ssl_call[ssl_try_count++] = 'y';
 				ssl_call[ssl_try_count++] = 'n';
 				break;
-			case 'a':
+			case SSLLBYTE_ALLOW:
 				ssl_call[ssl_try_count++] = 'n';
 				ssl_call[ssl_try_count++] = 'y';
 				break;
@@ -1407,11 +1450,11 @@ another_version_retry:
 					return 0;
 				}
 				if (PROTOCOL_63(ci))
-					strncpy(ci->protocol, PG62, sizeof(ci->protocol));
+					strncpy_null(ci->protocol, PG62, sizeof(ci->protocol));
 				else if (PROTOCOL_64(ci))
-					strncpy(ci->protocol, PG63, sizeof(ci->protocol));
+					strncpy_null(ci->protocol, PG63, sizeof(ci->protocol));
 				else 
-					strncpy(ci->protocol, PG64, sizeof(ci->protocol));
+					strncpy_null(ci->protocol, PG64, sizeof(ci->protocol));
 #ifdef	USE_SSPI
 				ssl_try_no = 0;
 			}
@@ -1508,8 +1551,8 @@ inolog("protocol=%s version=%d,%d\n", ci->protocol, self->pg_version_major, self
 			sock->pversion = PG_PROTOCOL_62;
 			SOCK_put_int(sock, htonl(4 + sizeof(StartupPacket6_2)), 4);
 			sp62.authtype = htonl(NO_AUTHENTICATION);
-			strncpy(sp62.database, ci->database, PATH_SIZE);
-			strncpy(sp62.user, ci->username, USRNAMEDATALEN);
+			strncpy_null(sp62.database, ci->database, PATH_SIZE);
+			strncpy_null(sp62.user, ci->username, USRNAMEDATALEN);
 			SOCK_put_n_char(sock, (char *) &sp62, sizeof(StartupPacket6_2));
 			SOCK_flush_output(sock);
 		}
@@ -1533,8 +1576,8 @@ inolog("protocol=%s version=%d,%d\n", ci->protocol, self->pg_version_major, self
 
 			sp.protoVersion = (ProtocolVersion) htonl(sock->pversion);
 
-			strncpy(sp.database, ci->database, SM_DATABASE);
-			strncpy(sp.user, ci->username, SM_USER);
+			strncpy_null(sp.database, ci->database, SM_DATABASE);
+			strncpy_null(sp.user, ci->username, SM_USER);
 
 			SOCK_put_n_char(sock, (char *) &sp, sizeof(StartupPacket));
 			SOCK_flush_output(sock);
@@ -1585,7 +1628,7 @@ inolog("protocol=%s version=%d,%d\n", ci->protocol, self->pg_version_major, self
 							goto sockerr_proc;
 					}
 					else
-						strncpy(ci->protocol, PG74REJECTED, sizeof(ci->protocol));
+						strncpy_null(ci->protocol, PG74REJECTED, sizeof(ci->protocol));
 				}
 				startPacketReceived = TRUE;
 			}
@@ -1788,7 +1831,7 @@ CC_connect(ConnectionClass *self, char password_req, char *salt_para)
 {
 	ConnInfo *ci = &(self->connInfo);
 	CSTR	func = "CC_connect";
-	char		ret, *saverr = NULL;
+	char		ret, *saverr = NULL, retsend;
 #ifndef	NOT_USE_LIBPQ
 	BOOL	call_libpq = FALSE;
 #endif /* NOT_USE_LIBPQ */
@@ -1799,7 +1842,7 @@ CC_connect(ConnectionClass *self, char password_req, char *salt_para)
 #ifndef	NOT_USE_LIBPQ
 	if (self->connInfo.username[0] == '\0')
 		call_libpq = TRUE;
-	else if (self->connInfo.sslmode[0] != 'd') 
+	else if (self->connInfo.sslmode[0] != SSLLBYTE_DISABLE) 
 #ifdef	USE_SSPI
 	{
 		if (0 == (SchannelService & self->svcs_allowed))
@@ -1851,7 +1894,7 @@ CC_connect(ConnectionClass *self, char password_req, char *salt_para)
 	 * function instead.
 	 */
 inolog("CC_send_settings\n");
-	CC_send_settings(self);
+	retsend = CC_send_settings(self);
 
 	if (CC_get_errornumber(self) > 0)
 		saverr = strdup(CC_get_errormsg(self));
@@ -1937,6 +1980,8 @@ cleanup:
 			CC_set_error(self, -1, saverr, func);
 		free(saverr);
 	}
+	if (1 == ret && 0 == retsend)
+		ret = 2;
 
 	return ret;
 }
@@ -2051,7 +2096,7 @@ CC_create_errormsg(ConnectionClass *self)
 	msg[0] = '\0';
 
 	if (CC_get_errormsg(self))
-		strncpy(msg, CC_get_errormsg(self), sizeof(msg));
+		strncpy_null(msg, CC_get_errormsg(self), sizeof(msg));
 
 	mylog("msg = '%s'\n", msg);
 
@@ -2128,6 +2173,37 @@ CC_get_error(ConnectionClass *self, int *number, char **message)
 }
 
 
+static int CC_close_eof_cursors(ConnectionClass *self)
+{
+	int	i, ccount = 0;
+	StatementClass	*stmt;
+	QResultClass	*res;
+
+	if (!self->ncursors)
+		return ccount;
+	CONNLOCK_ACQUIRE(self);
+	for (i = 0; i < self->num_stmts; i++)
+	{
+		if (stmt = self->stmts[i], NULL == stmt)
+			continue;
+		if (res = SC_get_Result(stmt), NULL == res)
+			continue;
+		if (NULL != QR_get_cursor(res) &&
+		    QR_is_withhold(res) &&
+		    QR_once_reached_eof(res))
+		{
+			if (QR_get_num_cached_tuples(res) >= QR_get_num_total_tuples(res) ||
+	    		    SQL_CURSOR_FORWARD_ONLY == stmt->options.cursor_type)
+			{
+				QR_close(res);
+				ccount++;
+			}
+		}
+	}
+	CONNLOCK_RELEASE(self);
+	return ccount;
+}
+
 static void CC_clear_cursors(ConnectionClass *self, BOOL on_abort)
 {
 	int	i;
@@ -2543,7 +2619,7 @@ inolog("send_query response_length=%d\n", response_length);
 
 					mylog("send_query: setting cmdbuffer = '%s'\n", cmdbuffer);
 
-					trim(cmdbuffer); /* get rid of trailing space */ 
+					my_trim(cmdbuffer); /* get rid of trailing space */ 
 					if (strnicmp(cmdbuffer, bgncmd, lenbgncmd) == 0)
 					{
 						CC_set_in_trans(self);
@@ -2587,7 +2663,7 @@ inolog("Discarded the first SAVEPOINT\n");
 						}
 						else
 						{
-							if (strnicmp(cmdbuffer, "COMMIT", 6) == 0)
+							if (strnicmp(cmdbuffer, cmtcmd, 6) == 0)
 								CC_on_commit(self);
 							else if (strnicmp(cmdbuffer, "END", 3) == 0)
 								CC_on_commit(self);
@@ -2661,6 +2737,8 @@ inolog("Discarded the first SAVEPOINT\n");
 					if (--empty_reqs == 0)
 						query_completed = TRUE;
 				}
+				else if (!beforeV2)
+					query_completed = TRUE;
 				break;
 			case 'E':
 				msg_truncated = handle_error_message(self, msgbuffer, sizeof(msgbuffer), res->sqlstate, "send_query", res);
@@ -3513,7 +3591,7 @@ CC_send_cancel_request(const ConnectionClass *conn)
 	crp.cp.backendPID = htonl(conn->be_pid);
 	crp.cp.cancelAuthCode = htonl(conn->be_key);
 
-	while (send(tmpsock, (char *) &crp, sizeof(crp), 0) != (int) sizeof(crp))
+	while (send(tmpsock, (char *) &crp, sizeof(crp), SEND_FLAG) != (int) sizeof(crp))
 	{
 		if (SOCK_ERRNO != EINTR)
 		{
@@ -3524,7 +3602,7 @@ CC_send_cancel_request(const ConnectionClass *conn)
 	}
 	if (ret)
 	{
-		while (recv(tmpsock, (char *) &crp, 1, 0) < 0)
+		while (recv(tmpsock, (char *) &crp, 1, RECV_FLAG) < 0)
 		{
 			if (EINTR != SOCK_ERRNO)
 				break;
@@ -3553,6 +3631,7 @@ int	CC_mark_a_object_to_discard(ConnectionClass *conn, int type, const char *pla
 
 	return 1;
 }
+
 int	CC_discard_marked_objects(ConnectionClass *conn)
 {
 	int	i, cnt;
@@ -3604,7 +3683,8 @@ inolog("sock=%p\n", sock);
 
 	if (!(conninfo = protocol3_opts_build(self)))
 	{
-		CC_set_error(self, CONN_OPENDB_ERROR, "Couldn't allcate conninfo", func);
+		if (CC_get_errornumber(self) <= 0)
+			CC_set_error(self, CONN_OPENDB_ERROR, "Couldn't allcate conninfo", func);
 		goto cleanup1;
 	}
 	pqconn = CALL_PQconnectdb(conninfo, &libpqLoaded);
@@ -3663,17 +3743,17 @@ if (TRUE)
 		ConnInfo	*ci = &self->connInfo;
 
 		sock->pversion = PG_PROTOCOL_74;
-		strncpy(ci->protocol, PG74, sizeof(ci->protocol));
+		strncpy_null(ci->protocol, PG74, sizeof(ci->protocol));
 		pversion = PQprotocolVersion(pqconn);
 		switch (pversion)
 		{
 			case 2:
 				sock->pversion = PG_PROTOCOL_64;
-				strncpy(ci->protocol, PG64, sizeof(ci->protocol));
+				strncpy_null(ci->protocol, PG64, sizeof(ci->protocol));
 				break;
 		}
 	}
-	mylog("procotol=%s\n", self->connInfo.protocol);
+	mylog("protocol=%s\n", self->connInfo.protocol);
 	{
 		int pversion;
 		const char *conforming_strings;
