@@ -10,6 +10,7 @@
 #define __CONNECTION_H__
 
 #include "psqlodbc.h"
+#include <time.h>
 
 #include <stdlib.h>
 #include <string.h>
@@ -85,6 +86,8 @@ enum
 
 /* AutoCommit functions */
 #define CC_is_in_autocommit(x)		(x->transact_status & CONN_IN_AUTOCOMMIT)
+#define CC_does_autocommit(x) (CONN_IN_AUTOCOMMIT == ((x)->transact_status & (CONN_IN_AUTOCOMMIT | CONN_IN_MANUAL_TRANSACTION)))
+#define CC_loves_visible_trans(x) ((0 == ((x)->transact_status & CONN_IN_AUTOCOMMIT)) || (0 != ((x)->transact_status & CONN_IN_MANUAL_TRANSACTION)))
 
 /* Transaction in/not functions */
 #define CC_set_in_trans(x)	(x->transact_status |= CONN_IN_TRANSACTION)
@@ -283,7 +286,7 @@ typedef struct
 	char		conn_settings[LARGE_REGISTRY_LEN];
 	char		protocol[SMALL_REGISTRY_LEN];
 	char		port[SMALL_REGISTRY_LEN];
-	char		sslmode[SMALL_REGISTRY_LEN];
+	char		sslmode[16];
 	char		onlyread[SMALL_REGISTRY_LEN];
 	char		fake_oid_index[SMALL_REGISTRY_LEN];
 	char		show_oid_column[SMALL_REGISTRY_LEN];
@@ -371,11 +374,24 @@ typedef struct
 struct col_info
 {
 	Int2		num_reserved_cols;
+	Int2		refcnt;
 	QResultClass	*result;
 	pgNAME		schema_name;
 	pgNAME		table_name;
 	OID		table_oid;
+	time_t		acc_time;
 };
+#define free_col_info_contents(coli) \
+{ \
+	if (NULL != coli->result) \
+		QR_Destructor(coli->result); \
+	coli->result = NULL; \
+	NULL_THE_NAME(coli->schema_name); \
+	NULL_THE_NAME(coli->table_name); \
+	coli->table_oid = 0; \
+	coli->refcnt = 0; \
+	coli->acc_time = 0; \
+}
 #define col_info_initialize(coli) (memset(coli, 0, sizeof(COL_INFO)))
 
  /* Translation DLL entry points */
