@@ -371,7 +371,7 @@ inolog("prepare_before_exec=%d srv=%d\n", prepare_before_exec, conn->connInfo.us
 		for (curres = res; !curres->num_fields; curres = curres->next)
 			;
 		SC_set_Curres(stmt, curres);
-		if (CC_is_in_autocommit(conn))
+		if (CC_does_autocommit(conn))
 		{
 			if (issued_begin)
 				CC_commit(conn);
@@ -1091,9 +1091,7 @@ PGAPI_Transact(
 {
 	CSTR func = "PGAPI_Transact";
 	ConnectionClass *conn;
-	QResultClass *res;
-	char		ok,
-			   *stmt_string;
+	char		ok;
 	int			lf;
 
 	mylog("entering %s: hdbc=%p, henv=%p\n", func, hdbc, henv);
@@ -1125,24 +1123,19 @@ PGAPI_Transact(
 
 	conn = (ConnectionClass *) hdbc;
 
-	if (fType == SQL_COMMIT)
-		stmt_string = "COMMIT";
-	else if (fType == SQL_ROLLBACK)
-		stmt_string = "ROLLBACK";
-	else
+	if (fType != SQL_COMMIT &&
+	    fType != SQL_ROLLBACK)
 	{
 		CC_set_error(conn, CONN_INVALID_ARGUMENT_NO, "PGAPI_Transact can only be called with SQL_COMMIT or SQL_ROLLBACK as parameter", func);
 		return SQL_ERROR;
 	}
 
 	/* If manual commit and in transaction, then proceed. */
-	if (!CC_is_in_autocommit(conn) && CC_is_in_trans(conn))
+	if (CC_loves_visible_trans(conn) && CC_is_in_trans(conn))
 	{
-		mylog("PGAPI_Transact: sending on conn %d '%s'\n", conn, stmt_string);
+		mylog("PGAPI_Transact: sending on conn %p '%d'\n", conn, fType);
 
-		res = CC_send_query(conn, stmt_string, NULL, 0, NULL);
-		ok = QR_command_maybe_successful(res);
-		QR_Destructor(res);
+		ok = (SQL_COMMIT == fType) ? CC_commit(conn) : CC_abort(conn);
 		if (!ok)
 		{
 			/* error msg will be in the connection */
@@ -1378,7 +1371,7 @@ PGAPI_ParamData(
 		odbc_lo_close(conn, estmt->lobj_fd);
 
 		/* commit transaction if needed */
-		if (!CC_cursor_count(conn) && CC_is_in_autocommit(conn))
+		if (!CC_cursor_count(conn) && CC_does_autocommit(conn))
 		{
 			if (!CC_commit(conn))
 			{
