@@ -15,6 +15,9 @@
 #include <errno.h>
 #endif /* WIN32 */
 #ifndef NOT_USE_LIBPQ
+#ifdef	RESET_CRYPTO_CALLBACKS
+#include <openssl/ssl.h>
+#endif /* RESET_CRYPTO_CALLBACKS */
 #include <libpq-fe.h>
 #endif /* NOT_USE_LIBPQ */
 #include "loadlib.h"
@@ -26,6 +29,9 @@
 #ifndef	NOT_USE_LIBPQ
 #pragma comment(lib, "libpq")
 #pragma comment(lib, "ssleay32")
+#ifdef	RESET_CRYPTO_CALLBACKS
+#pragma comment(lib, "libeay32")
+#endif /* RESET_CRYPTO_CALLBACKS */
 #endif /* NOT_USE_LIBPQ */
 #ifdef	_HANDLE_ENLIST_IN_DTC_
 #ifdef	UNICODE_SUPPORT
@@ -40,6 +46,9 @@
 #ifndef	NOT_USE_LIBPQ
 #pragma comment(linker, "/Delayload:libpq.dll")
 #pragma comment(linker, "/Delayload:ssleay32.dll")
+#ifdef	RESET_CRYPTO_CALLBACKS
+#pragma comment(linker, "/Delayload:libeay32.dll")
+#endif /* RESET_CRYPTO_CALLBACKS */
 #endif /* NOT_USE_LIBPQ */
 #ifdef	UNICODE_SUPPORT
 #pragma comment(linker, "/Delayload:pgenlist.dll")
@@ -49,9 +58,9 @@
 #pragma comment(linker, "/Delay:UNLOAD")
 #endif /* _MSC_VER */
 #endif /* _MSC_VER */
+
 #if defined(DYNAMIC_LOAD)
 #define	WIN_DYN_LOAD
-CSTR	libpq = "libpq";
 CSTR	libpqdll = "LIBPQ.dll";
 #ifdef	UNICODE_SUPPORT
 CSTR	pgenlist = "pgenlist";
@@ -66,6 +75,12 @@ CSTR	pgenlistdll = "PGENLISTA.dll";
 #endif /* DYNAMIC_LOAD */
 #endif /* WIN32 */
 
+#ifndef	NOT_USE_LIBPQ
+CSTR	libpq = "libpq";
+CSTR	checkproc = "PQconninfoParse";
+static int	sslverify_available = -1;
+#endif /* NOT_USE_LIBPQ */
+
 #if defined(_MSC_DELAY_LOAD_IMPORT)
 static BOOL	loaded_libpq = FALSE, loaded_ssllib = FALSE;
 static BOOL	loaded_pgenlist = FALSE;
@@ -115,7 +130,7 @@ DliErrorHook(unsigned	dliNotify,
 	int	i;
 	static const char * const libarray[] = {"libssl32", "ssleay32"};
 
-	mylog("Dli%sHook Notify=%d %p\n", (dliFailLoadLib == dliNotify || dliFailGetProc == dliNotify) ? "Error" : "Notify", dliNotify, pdli);
+	mylog("Dli%sHook %s Notify=%d\n", (dliFailLoadLib == dliNotify || dliFailGetProc == dliNotify) ? "Error" : "Notify", NULL != pdli->szDll ? pdli->szDll : pdli->dlp.szProcName, dliNotify);
 	switch (dliNotify)
 	{
 		case dliNotePreLoadLibrary:
@@ -129,6 +144,17 @@ DliErrorHook(unsigned	dliNotify,
 			{
 				if (hmodule = MODULE_load_from_psqlodbc_path(libpq), NULL == hmodule)
 					hmodule = LoadLibrary(libpq);
+#ifndef	NOT_USE_LIBPQ
+				if (sslverify_available < 0)
+				{
+					if (NULL == hmodule)
+						sslverify_available = FALSE;
+					else if (NULL == GetProcAddress(hmodule, checkproc))
+						sslverify_available = FALSE;
+					else
+						sslverify_available = TRUE;
+				}
+#endif /* NOT_USE_LIBPQ */
 			}
 			else if (_strnicmp(pdli->szDll, pgenlist, strlen(pgenlist)) == 0)
 			{
@@ -172,6 +198,17 @@ void CleanupDelayLoadedDLLs(void)
 	/* The dll names are case sensitive for the unload helper */
 	if (loaded_libpq)
 	{
+#ifdef	RESET_CRYPTO_CALLBACKS
+		/*
+		 *	May be needed to avoid crash on exit
+		 *	when libpq doesn't reset the callbacks.
+		 */
+		CRYPTO_set_locking_callback(NULL);
+		CRYPTO_set_id_callback(NULL);
+		mylog("passed RESET_CRYPTO_CALLBACKS\n");
+#else
+		mylog("not passed RESET_CRYPTO_CALLBACKS\n");
+#endif /* RESET_CRYPTO_CALLBACKS */
 		success = (*func)(libpqdll);
 		mylog("%s unload success=%d\n", libpqdll, success);
 	}
@@ -195,6 +232,56 @@ void CleanupDelayLoadedDLLs(void)
 #endif	/* _MSC_DELAY_LOAD_IMPORT */
 
 #ifndef	NOT_USE_LIBPQ
+#if defined(_MSC_DELAY_LOAD_IMPORT)
+static int filter_env2encoding(int level)
+{
+	switch (level & 0xffff)
+	{
+		case ERROR_MOD_NOT_FOUND:
+		case ERROR_PROC_NOT_FOUND:
+			return EXCEPTION_EXECUTE_HANDLER;
+	}
+	return EXCEPTION_CONTINUE_SEARCH;
+}
+#endif /* _MSC_DELAY_LOAD_IMPORT */
+
+BOOL	ssl_verify_available(void)
+{
+	if (sslverify_available < 0)
+	{
+#if defined(_MSC_DELAY_LOAD_IMPORT)
+		__try {
+#if (_MSC_VER < 1300)
+			__pfnDliFailureHook = DliErrorHook;
+			__pfnDliNotifyHook = DliErrorHook;
+#else
+			__pfnDliFailureHook2 = DliErrorHook;
+			__pfnDliNotifyHook2 = DliErrorHook;
+#endif /* _MSC_VER */
+			PQenv2encoding();
+		}
+		__except (filter_env2encoding(GetExceptionCode())) {
+		}
+		if (sslverify_available < 0)
+			sslverify_available = 0;
+#else
+#ifdef HAVE_LIBLTDL
+		lt_dlhandle dlhandle = lt_dlopenext(libpq);
+
+		sslverify_available = 1;
+		if (NULL != dlhandle)
+		{
+			if (NULL == lt_dlsym(dlhandle, checkproc))
+				sslverify_available = 0;
+			lt_dlclose(dlhandle);
+		}
+#endif /* HAVE_LIBLTDL */
+#endif /* _MSC_DELAY_LOAD_IMPORT */
+	}
+
+	return (0 != sslverify_available);
+}
+
 void *CALL_PQconnectdb(const char *conninfo, BOOL *libpqLoaded)
 {
 	void *pqconn = NULL;
