@@ -120,6 +120,18 @@ SQLSMALLINT	sqlTypes[] = {
 #define	ALLOWED_C_BIGINT	SQL_C_CHAR
 #endif
 
+OID
+pg_true_type(const ConnectionClass *conn, OID type, OID basetype)
+{
+	if (0 == basetype)
+		return type;
+	else if (0 == type)
+		return basetype;
+	else if (type == conn->lobj_type)
+		return type;
+	return basetype;
+}
+
 OID
 sqltype_to_pgtype(StatementClass *stmt, SQLSMALLINT fSqlType)
 {
@@ -258,7 +270,7 @@ pgtype_to_concise_type(StatementClass *stmt, OID type, int col)
 	ConnectionClass	*conn = SC_get_conn(stmt);
 	ConnInfo	*ci = &(conn->connInfo);
 #if (ODBCVER >= 0x0300)
-	EnvironmentClass *env = (EnvironmentClass *) (conn->henv);
+	EnvironmentClass *env = (EnvironmentClass *) CC_get_env(conn);
 #endif /* ODBCVER */
 
 	switch (type)
@@ -437,7 +449,7 @@ pgtype_to_ctype(StatementClass *stmt, OID type)
 	ConnectionClass	*conn = SC_get_conn(stmt);
 	ConnInfo	*ci = &(conn->connInfo);
 #if (ODBCVER >= 0x0300)
-	EnvironmentClass *env = (EnvironmentClass *) (conn->henv);
+	EnvironmentClass *env = (EnvironmentClass *) CC_get_env(conn);
 #endif /* ODBCVER */
 
 	switch (type)
@@ -505,7 +517,8 @@ pgtype_to_ctype(StatementClass *stmt, OID type)
 #endif /* UNICODE_SUPPORT */
 		case PG_TYPE_UUID:
 #if (ODBCVER >= 0x0350)
-			return SQL_C_GUID;
+			if (!conn->ms_jet)
+				return SQL_C_GUID;
 #endif /* ODBCVER */
 			return SQL_C_CHAR;
 
@@ -890,6 +903,8 @@ pgtype_column_size(StatementClass *stmt, OID type, int col, int handle_unknown_s
 	ConnectionClass *conn = SC_get_conn(stmt);
 	ConnInfo	*ci = &(conn->connInfo);
 
+	if (handle_unknown_size_as == UNKNOWNS_AS_DEFAULT)
+		handle_unknown_size_as = ci->drivers.unknown_sizes; 
 	switch (type)
 	{
 		case PG_TYPE_CHAR:
@@ -1033,6 +1048,11 @@ pgtype_display_size(StatementClass *stmt, OID type, int col, int handle_unknown_
 		case PG_TYPE_FLOAT8:
 			return 22;
 
+#if (ODBCVER >= 0x0350)
+		case PG_TYPE_UUID:
+			return 36;
+#endif /* ODBCVER */
+
 			/* Character types use regular precision */
 		default:
 			return pgtype_column_size(stmt, type, col, handle_unknown_size_as);
@@ -1084,6 +1104,9 @@ pgtype_buffer_length(StatementClass *stmt, OID type, int col, int handle_unknown
 		case PG_TYPE_TIMESTAMP_NO_TMZONE:
 			return 16;		/* sizeof(TIMESTAMP_STRUCT) */
 
+		case PG_TYPE_UUID:
+			return 16;		/* sizeof(SQLGUID) */
+
 			/* Character types use the default precision */
 		case PG_TYPE_VARCHAR:
 		case PG_TYPE_BPCHAR:
@@ -1160,22 +1183,22 @@ pgtype_desclength(StatementClass *stmt, OID type, int col, int handle_unknown_si
  *	Transfer octet length.
  */
 Int4
-pgtype_transfer_octet_length(StatementClass *stmt, OID type, int col, int handle_unknown_size_as)
+pgtype_transfer_octet_length(StatementClass *stmt, OID type, int column_size)
 {
 	ConnectionClass	*conn = SC_get_conn(stmt);
 
 	int	coef = 1;
-	Int4	prec = pgtype_column_size(stmt, type, col, handle_unknown_size_as), maxvarc;
+	Int4	maxvarc;
 	switch (type)
 	{
 		case PG_TYPE_VARCHAR:
 		case PG_TYPE_BPCHAR:
 		case PG_TYPE_TEXT:
-			if (SQL_NO_TOTAL == prec)
-				return prec;
+			if (SQL_NO_TOTAL == column_size)
+				return column_size;
 #ifdef	UNICODE_SUPPORT
 			if (CC_is_in_unicode_driver(conn))
-				return prec * WCLEN;
+				return column_size * WCLEN;
 #endif /* UNICODE_SUPPORT */
 			/* after 7.2 */
 			if (PG_VERSION_GE(conn, 7.2))
@@ -1184,16 +1207,16 @@ pgtype_transfer_octet_length(StatementClass *stmt, OID type, int col, int handle
 				/* CR -> CR/LF */
 				coef = 2;
 			if (coef == 1)
-				return prec;
+				return column_size;
 			maxvarc = conn->connInfo.drivers.max_varchar_size;
-			if (prec <= maxvarc && prec * coef > maxvarc)
+			if (column_size <= maxvarc && column_size * coef > maxvarc)
 				return maxvarc;
-			return coef * prec;
+			return coef * column_size;
 		case PG_TYPE_BYTEA:
-			return prec;
+			return column_size;
 		default:
 			if (type == conn->lobj_type)
-				return prec;
+				return column_size;
 	}
 	return -1;
 }
@@ -1588,6 +1611,14 @@ sqltype_to_default_ctype(const ConnectionClass *conn, SQLSMALLINT sqltype)
 			return SQL_C_TYPE_TIMESTAMP;
 #endif /* ODBCVER */
 
+#if (ODBCVER >= 0x0350)
+		case SQL_GUID:
+			if (conn->ms_jet)
+				return SQL_C_CHAR;
+			else
+				return SQL_C_GUID;
+#endif /* ODBCVER */
+
 		default:
 			/* should never happen */
 			return SQL_C_CHAR;
@@ -1647,6 +1678,11 @@ ctype_length(SQLSMALLINT ctype)
 #endif /* ODBCVER */
 			return sizeof(TIMESTAMP_STRUCT);
 
+#if (ODBCVER >= 0x0350)
+		case SQL_C_GUID:
+			return sizeof(SQLGUID);
+#endif /* ODBCVER */
+
 		case SQL_C_BINARY:
 		case SQL_C_CHAR:
 #ifdef	UNICODE_SUPPORT
