@@ -5,7 +5,7 @@
  *
  * Comments:		See "notice.txt" for copyright and license information.
  *
- * $Id: psqlodbc.h,v 1.123 2008/09/21 15:35:44 hinoue Exp $
+ * $Id: psqlodbc.h,v 1.131 2009/07/06 15:23:02 h-saito Exp $
  *
  */
 
@@ -119,11 +119,12 @@ typedef	UInt4	OID;
 #define	FORMAT_ULPTR	"%lu"	/* ULONG_PTR */
 #endif /* _WIN64 */
 #else
-#define	FORMAT_SIZE_T	"%xu"	/* size_t */	
-#define	FORMAT_SSIZE_T	"%xd"	/* ssize_t */
+#define	FORMAT_SIZE_T	"%zu"	/* size_t */	
+#define	FORMAT_SSIZE_T	"%zd"	/* ssize_t */
 #ifndef	HAVE_SSIZE_T
 typedef	long	ssize_t
 #endif /* HAVE_SSIZE_T */
+
 #if (SIZEOF_VOID_P == SIZEOF_LONG)
 typedef	long 	LONG_PTR;
 typedef	unsigned long 	ULONG_PTR;
@@ -159,6 +160,11 @@ typedef	unsigned long long ULONG_PTR;
 #ifndef	SQL_IS_LEN
 #define	SQL_IS_LEN	(-1000)
 #endif /* SQL_IS_LEN */
+#ifdef	HAVE_SIGNED_CHAR
+typedef	signed char	po_ind_t;
+#else
+typedef char	po_ind_t;
+#endif /* HAVE_SIGNED_CHAR */
 
 #ifndef WIN32
 #if !defined(WITH_UNIXODBC) && !defined(WITH_IODBC)
@@ -181,7 +187,12 @@ typedef double SDOUBLE;
 #define	FALSE	(BOOL)0
 #endif /* FALSE */
 #else
+#if (_MSC_VER >= 1400) && !defined(_WIN64)
+#define snprintf sprintf_s
+#define strncat(d, s, l) strcat_s(d, l, s)
+#else
 #define snprintf _snprintf
+#endif
 #ifndef strdup
 #define strdup _strdup
 #endif /* strdup */
@@ -470,8 +481,12 @@ const pthread_mutexattr_t *getMutexAttr(void);
 #define WCLEN sizeof(SQLWCHAR)
 SQLULEN	ucs2strlen(const SQLWCHAR *ucs2str);
 char	*ucs2_to_utf8(const SQLWCHAR *ucs2str, SQLLEN ilen, SQLLEN *olen, BOOL tolower);
-SQLULEN	utf8_to_ucs2_lf(const char * utf8str, SQLLEN ilen, BOOL lfconv, SQLWCHAR *ucs2str, SQLULEN buflen);
-#define	utf8_to_ucs2(utf8str, ilen, ucs2str, buflen) utf8_to_ucs2_lf(utf8str, ilen, FALSE, ucs2str, buflen)
+SQLULEN	utf8_to_ucs2_lf0(const char * utf8str, SQLLEN ilen, BOOL lfconv, SQLWCHAR *ucs2str, SQLULEN buflen);
+SQLULEN	utf8_to_ucs2_lf1(const char * utf8str, SQLLEN ilen, BOOL lfconv, SQLWCHAR *ucs2str, SQLULEN buflen);
+int	msgtowstr(const char *, const char *, int, LPWSTR, int);
+int	wstrtomsg(const char *, const LPWSTR, int, char *, int);
+#define	utf8_to_ucs2_lf(utf8str, ilen, lfconv, ucs2str, buflen) utf8_to_ucs2_lf0(utf8str, ilen, lfconv, ucs2str, buflen)
+#define	utf8_to_ucs2(utf8str, ilen, ucs2str, buflen) utf8_to_ucs2_lf0(utf8str, ilen, FALSE, ucs2str, buflen)
 #endif /* UNICODE_SUPPORT */
 
 
