@@ -237,8 +237,7 @@ PGAPI_FreeStmt(HSTMT hstmt,
 			if (STMT_EXECUTING == stmt->status)
 			{
 				SC_set_error(stmt, STMT_SEQUENCE_ERROR, "Statement is currently executing a transaction.", func);
-				return SQL_ERROR;		/* stmt may be executing a
-										 * transaction */
+				return SQL_ERROR; /* stmt may be executing a transaction */
 			}
 			/*
 			 * Free any cursors and discard any result info.
@@ -768,10 +767,10 @@ SC_recycle_statement(StatementClass *self)
 			 * start of a transaction. If so, we have to rollback that
 			 * transaction.
 			 */
-			if (!CC_is_in_autocommit(conn) && CC_is_in_trans(conn))
+			if (CC_loves_visible_trans(conn) && CC_is_in_trans(conn))
 			{
 				if (SC_is_pre_executable(self) && !SC_is_parse_tricky(self))
-					CC_abort(conn);
+					/* CC_abort(conn) */;
 			}
 			break;
 
@@ -856,7 +855,7 @@ inolog("%s statement=%p ommitted=0\n", func, self);
 void
 SC_scanQueryAndCountParams(const char *query, const ConnectionClass *conn,
 		Int4 *next_cmd, SQLSMALLINT * pcpar,
-		char *multi_st, char *proc_return)
+		po_ind_t *multi_st, po_ind_t *proc_return)
 {
 	CSTR func = "SC_scanQueryAndCountParams";
 	char	literal_quote = LITERAL_QUOTE, identifier_quote = IDENTIFIER_QUOTE, dollar_quote = DOLLAR_QUOTE;
@@ -865,7 +864,8 @@ SC_scanQueryAndCountParams(const char *query, const ConnectionClass *conn,
 	char	tchar, bchar, escape_in_literal = '\0';
 	char	in_literal = FALSE, in_identifier = FALSE,
 		in_dollar_quote = FALSE, in_escape = FALSE,
-		del_found = FALSE, multi = FALSE;
+		del_found = FALSE;
+	po_ind_t multi = FALSE;
 	SQLSMALLINT	num_p;
 	encoded_str	encstr;
 
@@ -1146,7 +1146,8 @@ static struct
 	{ STMT_OPTION_NOT_FOR_THE_DRIVER, "HYC00", "HYC00" },
 	{ STMT_FETCH_OUT_OF_RANGE, "HY106", "S1106" },
 	{ STMT_COUNT_FIELD_INCORRECT, "07002", "07002" },
-	{ STMT_INVALID_NULL_ARG, "HY009", "S1009" }
+	{ STMT_INVALID_NULL_ARG, "HY009", "S1009" },
+	{ STMT_NO_RESPONSE, "08S01", "08S01" }
 };
 
 static PG_ErrorInfo *
@@ -1185,7 +1186,7 @@ SC_create_errorinfo(const StatementClass *self)
 		}
 		if (NULL != res->message)
 		{
-			strncpy(msg, res->message, sizeof(msg));
+			strncpy_null(msg, res->message, sizeof(msg));
 			detailmsg = resmsg = TRUE;
 		}
 		if (msg[0])
@@ -1216,7 +1217,7 @@ SC_create_errorinfo(const StatementClass *self)
 			msg[pos++] = ';';
 			msg[pos++] = '\n';
 		}
-		strncpy(msg + pos, wmsg, sizeof(msg) - pos);
+		strncpy_null(msg + pos, wmsg, sizeof(msg) - pos);
 		ermsg = msg;
 		detailmsg = TRUE;
 	}
@@ -1732,9 +1733,19 @@ SC_execute(StatementClass *self)
 	is_in_trans = CC_is_in_trans(conn);
 	/* issue BEGIN ? */
 	issue_begin = TRUE;
-	if (self->internal || is_in_trans)
+	if (self->internal)
 		issue_begin = FALSE;
-	else if (CC_is_in_autocommit(conn) &&
+	else if (is_in_trans)
+	{
+		issue_begin = FALSE;
+		if (STMT_TYPE_START == self->statement_type &&
+		    CC_does_autocommit(conn))
+		{
+			CC_commit(conn);
+			is_in_trans = CC_is_in_trans(conn);
+		}
+	}
+	else if (CC_does_autocommit(conn) &&
 		 (!SC_is_fetchcursor(self)
 	    /* || SC_is_with_hold(self) thiw would lose the performance */
 		 ))
@@ -1821,7 +1832,7 @@ inolog("get_Result=%p %p %d\n", res, SC_get_Result(self), self->curr_param_resul
 		if (!(res = SendSyncAndReceive(self, self->curr_param_result ? res : NULL, "bind_and_execute")))
 		{
 			if (SC_get_errornumber(self) <= 0)
-				SC_set_error(self, STMT_EXEC_ERROR, "Could not receive the response, communication down ??", func);
+				SC_set_error(self, STMT_NO_RESPONSE, "Could not receive the response, communication down ??", func);
 			CC_on_abort(conn, CONN_DEAD);
 			goto cleanup;
 		}
@@ -1891,7 +1902,7 @@ inolog("get_Result=%p %p %d\n", res, SC_get_Result(self), self->curr_param_resul
 		{
 			if (!is_in_trans)
 				CC_set_in_manual_trans(conn);
-			if (!self->internal && CC_is_in_autocommit(conn) && !CC_is_in_manual_trans(conn))
+			if (!self->internal && CC_does_autocommit(conn))
 				CC_commit(conn);
 		}
 	}
@@ -2240,7 +2251,7 @@ RequestStart(StatementClass *stmt, ConnectionClass *conn, const char *func)
 		SC_set_error(stmt, STMT_INTERNAL_ERROR, emsg, func);
 		return FALSE;
 	}
-	if (!CC_is_in_trans(conn) && !CC_is_in_autocommit(conn))
+	if (!CC_is_in_trans(conn) && CC_loves_visible_trans(conn))
 	{
 		if (ret = CC_begin(conn), !ret)
 			return ret;
@@ -2293,7 +2304,7 @@ QResultClass *SendSyncAndReceive(StatementClass *stmt, QResultClass *res, const
 		id = SOCK_get_id(sock);
 		if ((SOCK_get_errcode(sock) != 0) || (id == EOF))
 		{
-			SC_set_error(stmt, CONNECTION_NO_RESPONSE, "No response rom the backend", func);
+			SC_set_error(stmt, STMT_NO_RESPONSE, "No response rom the backend", func);
 
 			mylog("%s: 'id' - %s\n", func, SC_get_errormsg(stmt));
 			CC_on_abort(conn, CONN_DEAD);
@@ -2529,7 +2540,8 @@ mylog("sta_pidx=%d end_pidx=%d num_p=%d\n", sta_pidx, end_pidx, num_params);
 	qlen = (SQL_NTS == qlen) ? strlen(query) : qlen; 
 	leng = strlen(plan_name) + 1 + qlen + 1 + pileng;
 	SOCK_put_int(sock, (Int4) (leng + 4), 4); /* length */
-inolog("parse leng=%d\n", leng);
+/* inolog("parse leng=%d\n", leng); */
+inolog("parse leng=" FORMAT_SIZE_T "\n", leng);
 	SOCK_put_string(sock, plan_name);
 	SOCK_put_n_char(sock, query, qlen);
 	SOCK_put_char(sock, '\0');
